﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Services.Hive.Common.ServiceContracts;
using HeuristicLab.Services.Hive.Common.DataTransfer;
using HeuristicLab.Services.Hive.Common;

namespace HeuristicLab.Clients.Hive.Slave.Tests {
  internal class MockHiveService : IHiveService {
    private static List<Job> jobs;
    private static List<JobData> jobDatas;
    private static List<HiveExperiment> hiveExperiments;

    static MockHiveService() {
      
      byte[] data = PersistenceUtil.Serialize(new MockJob(5000));

      jobs = new List<Job>();
      jobs.Add(new Job { Id = Guid.NewGuid(), JobState = JobState.Waiting, DateCreated = DateTime.Now, CoresNeeded = 1, MemoryNeeded = 0 });
      jobs.Add(new Job { Id = Guid.NewGuid(), JobState = JobState.Offline, DateCreated = DateTime.Now, CoresNeeded = 1, MemoryNeeded = 100, ParentJobId = jobs.First().Id });
      jobs.Add(new Job { Id = Guid.NewGuid(), JobState = JobState.Offline, DateCreated = DateTime.Now, CoresNeeded = 1, MemoryNeeded = 1024, ParentJobId = jobs.First().Id });
      jobs.Add(new Job { Id = Guid.NewGuid(), JobState = JobState.Offline, DateCreated = DateTime.Now, CoresNeeded = 1, MemoryNeeded = 4096, ParentJobId = jobs.First().Id });

      jobDatas = new List<JobData>();
      foreach (var job in jobs) {
        jobDatas.Add(new JobData() { JobId = job.Id, Data = data });
      }

      hiveExperiments = new List<HiveExperiment>();
      hiveExperiments.Add(new HiveExperiment() { Id = Guid.NewGuid(), Name = "Hive Exp 1", Description = "", RootJobId = jobs[0].Id });
    }

    #region Job Methods
    public Guid AddJob(Job job, JobData jobData) {
      job.Id = Guid.NewGuid();
      jobData.JobId = job.Id;
      jobs.Add(job);
      jobDatas.Add(jobData);
      return job.Id;
    }

    public Guid AddChildJob(Guid parentJobId, Job job, JobData jobData) {
      job.Id = Guid.NewGuid();
      job.ParentJobId = parentJobId;
      jobData.JobId = job.Id;
      jobs.Add(job);
      jobDatas.Add(jobData);
      return job.Id;
    }

    public Job GetJob(Guid jobId) {
      return jobs.Where(j => j.Id == jobId).SingleOrDefault();
    }

    public IEnumerable<Job> GetJobs() {
      return jobs;
    }

    public IEnumerable<LightweightJob> GetLightweightJobs(IEnumerable<Guid> jobIds) {
      return jobs.Select(j => new LightweightJob(j)); // not real
    }

    public IEnumerable<LightweightJob> GetLightweightChildJobs(Guid? parentJobId, bool recursive, bool includeParent) {
      return jobs.Where(j => j.ParentJobId.HasValue).Select(j => new LightweightJob(j));
    }

    public JobData GetJobData(Guid jobId) {
      return jobDatas.Where(jd => jd.JobId == jobId).SingleOrDefault();
    }

    public void UpdateJob(Job jobDto, JobData jobDataDto) {
      // do nothing
    }

    public void DeleteChildJobs(Guid parentJobId) {
      // do nothing
    }
    #endregion

    #region JobControl Methods
    public Job AquireJob(Guid slaveId) {
      var job = jobs.First();
      job.SlaveId = slaveId;
      return job;
    }

    public void StopJob(Guid jobId) {
      // do nothing
    }

    public void PauseJob(Guid jobId) {
      // do nothing
    }
    #endregion

    #region HiveExperiment Methods
    public HiveExperiment GetHiveExperiment(Guid id) {
      return hiveExperiments.Where(he => he.Id == id).SingleOrDefault();
    }

    public IEnumerable<HiveExperiment> GetHiveExperiments() {
      return hiveExperiments;
    }

    public Guid AddHiveExperiment(HiveExperiment hiveExperimentDto) {
      hiveExperimentDto.Id = Guid.NewGuid();
      hiveExperiments.Add(hiveExperimentDto);
      return hiveExperimentDto.Id;
    }

    public void UpdateHiveExperiment(HiveExperiment hiveExperimentDto) {
      // do nothing
    }

    public void DeleteHiveExperiment(Guid hiveExperimentId) {
      // do nothing
    }
    #endregion

    #region Login Methods
    public void Hello(Guid slaveId, string name, int cores, int memory) {
      // do nothing
    }

    public void GoodBye() {
      // do nothing
    }

    public List<MessageContainer> Heartbeat(Heartbeat heartbeat) {
      // todo
      return new List<MessageContainer>();
    }
    #endregion

    #region Plugin Methods
    public Guid AddPlugin(Plugin plugin, List<PluginData> pluginData) {
      // todo
      return Guid.NewGuid();
    }

    public IEnumerable<Plugin> GetPlugins() {
      // todo
      return new List<Plugin>();
    }

    public IEnumerable<PluginData> GetPluginDatas(List<Guid> pluginIds) {
      // todo
      return new List<PluginData>();
    }
    #endregion

    #region Slave Methods
    public Guid AddSlave(Services.Hive.Common.DataTransfer.Slave slave) {
      throw new NotImplementedException();
    }

    public Guid AddSlaveGroup(SlaveGroup slaveGroup) {
      throw new NotImplementedException();
    }

    public IEnumerable<Services.Hive.Common.DataTransfer.Slave> GetSlaves() {
      throw new NotImplementedException();
    }

    public IEnumerable<SlaveGroup> GetSlaveGroups() {
      throw new NotImplementedException();
    }

    public void DeleteSlaveGroup(Guid clientGroupId) {
      throw new NotImplementedException();
    }

    public void AddResourceToGroup(Guid slaveGroupId, Resource resource) {
      throw new NotImplementedException();
    }

    public void RemoveResourceFromGroup(Guid slaveGroupId, Guid resourceId) {
      throw new NotImplementedException();
    }

    public void UpdateSlave(Services.Hive.Common.DataTransfer.Slave slave) {
      throw new NotImplementedException();
    }
    #endregion
  }
}
